from os import environ

import requests

APIKEY = environ.get("YOUTUBE_APIKEY")


def get_external_id(htmltext: str):
    split = htmltext.split("channel_id=")
    if len(split) > 1:
        return split[1].split('"')[0]
    return None


def get_defacto_upload_id(external_id: str):
    return "UU" + external_id.lstrip("UC")


def get_upload_id(external_id):
    url = "https://www.googleapis.com/youtube/v3/channels"
    body = requests.get(url, params={
        "part": "contentDetails",
        "id": external_id,
        "key": APIKEY
    }).json()

    return body["items"][0]["contentDetails"]["relatedPlaylists"]["uploads"]


def get_all_videos(upload_id: str):
    url = "https://www.googleapis.com/youtube/v3/playlistItems"
    params = {
        "playlistId": upload_id,
        "key": APIKEY,
        "part": "snippet",
        "maxResults": 50
    }

    while True:
        body = requests.get(url, params=params).json()

        for item in body["items"]:
            snippet = item["snippet"]
            video_id = snippet["resourceId"]["videoId"]
            yield {
                "fileid": video_id,
                "filename": snippet["title"],
                "sourceurl": f"https://youtube.com/watch/{video_id}",
                "thumbnailurl": snippet["thumbnails"]["medium"]["url"]
            }

        next_page_token = body.get("nextPageToken")

        if not next_page_token:
            break

        params["pageToken"] = next_page_token


def scrape_youtubeapi(url):
    r = requests.get(url)
    external_id = get_external_id(r.text)
    upload_id = get_defacto_upload_id(external_id)
    yield from get_all_videos(upload_id)



from typing import Generator

from commmons import head, md5
from lxml.html import HtmlElement

from hypersquirrel.util import html_from_url_with_headers


def find_files(root: HtmlElement, li_class: str):
    for li in root.xpath(f"//li[@class='{li_class}']"):
        a = head(li.xpath(".//a"))
        img = head(li.xpath(".//img"))
        if img is None or a is None:
            continue

        thumbnail_url = img.attrib["src"]
        title = img.attrib["alt"]
        href = a.attrib["href"]

        yield {
            "fileid": f"drts{md5(href)}",
            "sourceurl": href,
            "filename": title,
            "thumbnailurl": thumbnail_url
        }


def vids(root: HtmlElement):
    yield from find_files(root, "thumi")


def pics(root: HtmlElement):
    for file in find_files(root, "thumbphoto"):
        source_url = file["sourceurl"]
        if "gallery" in source_url:
            yield {
                **file,
                "sourceurl": f"vpr://{source_url}"
            }


def scrape_drts(url: str) -> Generator[dict, None, None]:
    root = html_from_url_with_headers(url)
    yield from vids(root)
    yield from pics(root)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hypersquirrel",
    version="0.8",
    author="vka",
    description="Scrapes posts from various websites",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(exclude=("test",)),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=["requests", "vripper", "htmllistparse", "commmons", "feedparser"]
)
